/*
 * $Header: /cvs/CVS_LEBA/external/openmap/openmap/src/openmap/com/bbn/openmap/plugin/wms/ServerPanel.java,v 1.3 2006/04/27 05:58:25 pitek Exp $
 *
 */

package com.bbn.openmap.plugin.wms;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;
import com.bbn.openmap.util.PropUtils;
import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

/**
 * @author tomk (http://www.ctm.gdynia.pl)
 */
public class ServerPanel extends JPanel {

    //~ Instance variables ---------------------------------------------------------------------------------------------

    protected I18n i18n = Environment.getI18n();
    
    JPanel propertyPanel;
    ActionListener closeAction;
    JTextField jTextFieldServer;
    JTextField jTextFieldVersion;
    JTextField jTextFieldVendorNames;
    JTextField jTextFieldVendorValues;
    WMSPlugIn wmsPlugin;

    //~ Constructors ---------------------------------------------------------------------------------------------------

    /**
     * Creates a new instance of ServerPanel
     *
     * @param wmsServer
     * @param closeAction
     */
    public ServerPanel(WMSPlugIn wmsPlugin, ActionListener closeAction) {        
        this.closeAction = closeAction;
        this.wmsPlugin = wmsPlugin;
        createGUI();
    }

    //~ Methods --------------------------------------------------------------------------------------------------------

    /**
     * @param gridbag
     * @param c
     * @param labelText
     * @param y
     */
    private void addLabel(GridBagLayout gridbag, GridBagConstraints c, String labelText, int y) {
        JLabel label = new JLabel(labelText + ":");
        label.setHorizontalAlignment(SwingConstants.RIGHT);
        c.gridy = y;
        gridbag.setConstraints(label, c);
        propertyPanel.add(label);
    }

    /**
     * @param gridbag
     * @param c
     * @param text
     * @param y
     */
    private JTextField addTextField(GridBagLayout gridbag, GridBagConstraints c, String text, int y) {
        JTextField jText = new JTextField(text);
        c.gridy = y;
        gridbag.setConstraints(jText, c);
        propertyPanel.add(jText);
        return jText;
    }

    /**
     */
    void createGUI() {
        JButton doneButton = null;
        JButton cancelButton = null;
        setLayout(new BorderLayout());
        propertyPanel = new JPanel();

        GridBagLayout gridbag = new GridBagLayout();

        GridBagConstraints cWest = new GridBagConstraints();
        cWest.insets = new Insets(2, 10, 2, 10);
        cWest.gridx = 0;
        cWest.gridy = 0;
        cWest.anchor = GridBagConstraints.EAST;
        cWest.fill = GridBagConstraints.NONE;

        GridBagConstraints cEast = new GridBagConstraints();
        cEast.insets = new Insets(2, 10, 2, 10);
        cEast.gridx = 1;
        cEast.gridy = 0;
        cEast.weightx = 1f;
        cEast.fill = GridBagConstraints.HORIZONTAL;
        cEast.anchor = GridBagConstraints.WEST;
        String s = i18n.get(ServerPanel.class, "wmsServer", "WMS Server");
        addLabel(gridbag, cWest, s, 0);
        s = i18n.get(ServerPanel.class, "wmsVersion", "WMS Version");
        addLabel(gridbag, cWest, s, 1);
        s = i18n.get(ServerPanel.class, "vendorNames", "Vendor specific names");
        addLabel(gridbag, cWest, s, 2);
        s = i18n.get(ServerPanel.class, "vendorValues", "Vendor specific values");
        addLabel(gridbag, cWest, s, 3);
        
        final Properties props = new Properties();
        wmsPlugin.getProperties(props);
        final String prefix = PropUtils.getScopedPropertyPrefix(wmsPlugin);
        
        jTextFieldServer = addTextField(gridbag, cEast, props.getProperty(prefix + WMSPlugIn.WMSServerProperty), 0);
        jTextFieldVersion = addTextField(gridbag, cEast, props.getProperty(prefix + WMSPlugIn.WMSVersionProperty), 1);
        jTextFieldVendorNames = addTextField(gridbag, cEast, props.getProperty(prefix + WMSPlugIn.VendorSpecificNamesProperty), 2);
        jTextFieldVendorValues = addTextField(gridbag, cEast, props.getProperty(prefix + WMSPlugIn.VendorSpecificValuesProperty), 3);

        propertyPanel.setLayout(gridbag);

        add(propertyPanel, BorderLayout.CENTER);

        JPanel buttons = new JPanel();
        doneButton = new JButton(i18n.get(ServerPanel.class, "ok", "OK"));
        cancelButton = new JButton(i18n.get(ServerPanel.class, "cancel", "Cancel"));

        doneButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent ae) {
                    Properties props = new Properties();
                    wmsPlugin.getProperties(props);
                    props.setProperty(prefix + WMSPlugIn.WMSServerProperty, jTextFieldServer.getText());
                    props.setProperty(prefix + WMSPlugIn.WMSVersionProperty, jTextFieldVersion.getText());
                    props.setProperty(prefix + WMSPlugIn.VendorSpecificNamesProperty, jTextFieldVendorNames.getText());
                    props.setProperty(prefix + WMSPlugIn.VendorSpecificValuesProperty, jTextFieldVendorValues.getText());
                    wmsPlugin.setProperties(prefix, props);
                    wmsPlugin.getCapabilities();
                    wmsPlugin.refresh();
                    closeAction.actionPerformed(null);
                }
            });
        cancelButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent ae) {
                    closeAction.actionPerformed(null);
                }
            });

        buttons.add(doneButton);
        buttons.add(cancelButton);
        buttons.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));
        add(buttons, BorderLayout.SOUTH);
        validate();                
    }
}
