/*
 * $Header: /cvs/CVS_LEBA/external/openmap/openmap/src/openmap/com/bbn/openmap/plugin/wms/WmsLayersPanel.java,v 1.4 2006/04/27 05:58:25 pitek Exp $
 *
 */

package com.bbn.openmap.plugin.wms;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.net.URL;
import java.util.Collections;
import java.util.List;

import javax.swing.DefaultCellEditor;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;
import com.bbn.openmap.wms.DefaultWmsLayerStyle;

/**
 * Supports selecting layers to display. Display some info about layers in the table.
 * You can also choose style. 
 * 
 * @todo Renderer for style column
 * 
 * @author tomk (http://www.ctm.gdynia.pl)
 */
public class WmsLayersPanel extends javax.swing.JPanel {

    //~ Enums ----------------------------------------------------------------------------------------------------------

    /**
     * Enum
     */
    enum COLUMNS {
        VISIBLE, TITLE, NAME, STYLE
    }

    //~ Instance variables ---------------------------------------------------------------------------------------------

    protected I18n i18n = Environment.getI18n();

    private javax.swing.JScrollPane jspTable;
    private javax.swing.JTable jtLayers;
    private JButton jButtonDown;
    private JButton jButtonUp;
    private JButton jButtonInvert;
    // End of variables declaration

    private WmsTableModel tableModel;

    //~ Constructors ---------------------------------------------------------------------------------------------------

    /**
     * Creates new form WmsPanelListPanel
     *
     * @param wmsList
     */
    public WmsLayersPanel(List<BasicWmsLayer> wmsList) {
        initComponents(wmsList);
    }

    //~ Methods --------------------------------------------------------------------------------------------------------

    /**
     * @param wmsList
     */
    private void initComponents(List<BasicWmsLayer> wmsList) {
        this.setBackground(Color.RED);
        jspTable = new javax.swing.JScrollPane();
        jtLayers = new javax.swing.JTable();
        jtLayers.getTableHeader().setReorderingAllowed(false);
        setLayout(new java.awt.BorderLayout());

        jspTable.setViewportView(jtLayers);
        jspTable.setPreferredSize(new Dimension(400, 150));
        add(jspTable, java.awt.BorderLayout.CENTER);

        tableModel = new WmsTableModel(wmsList);
        jtLayers.setModel(tableModel);
        jtLayers.setColumnModel(new WmsColumnModel());
        jtLayers.createDefaultColumnsFromModel();
        jtLayers.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        TableColumn column = jtLayers.getColumnModel().getColumn(COLUMNS.STYLE.ordinal());
        column.setCellEditor(new ComboBoxEditor(wmsList));

        JPanel bottomPanel = new JPanel();

        jButtonDown = createButton(bottomPanel, "down.gif", "moveLayerDown", "Move layer down");
        jButtonUp = createButton(bottomPanel, "up.gif", "moveLayerUp", "Move layer up");
        jButtonInvert = createButton(bottomPanel, "invert.gif", "invertLayers", "Invert layers");

        bottomPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
        add(bottomPanel, java.awt.BorderLayout.SOUTH);

        ListSelectionListener selectionListener = new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (e.getValueIsAdjusting()) {
                    return;
                }
                setButtonStates();
            }
        };
        jtLayers.getSelectionModel().addListSelectionListener(selectionListener);
        setButtonStates();
    }

    /**
     */
    private void setButtonStates() {
        int row = jtLayers.getSelectedRow();
        jButtonDown.setEnabled((row >= 0) && (row < (jtLayers.getRowCount() - 1)));
        jButtonUp.setEnabled((row > 0) && (row < jtLayers.getRowCount()));
        jButtonInvert.setEnabled(jtLayers.getRowCount() > 0);
    }

    /**
     * @param container
     * @param iconFile
     * @param i18nKey
     * @param defaultLabel
     * @return
     */
    private JButton createButton(Container container, String iconFile, String i18nKey, String defaultLabel) {
        JButton jButton = new JButton();
        jButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    if (evt.getSource() == jButtonDown) {
                        moveDown();
                    } else if (evt.getSource() == jButtonUp) {
                        moveUp();
                    } else if (evt.getSource() == jButtonInvert) {
                        invert();
                    }
                }
            });
        container.add(jButton);
        URL url = WmsLayersPanel.class.getResource(iconFile);
        if (url != null) {
            ImageIcon icon = new ImageIcon(url);
            jButton.setIcon(icon);
        }
        jButton.setIconTextGap(0);
        jButton.setMargin(new java.awt.Insets(2, 2, 2, 2));
        jButton.setToolTipText(i18n.get(WmsLayersPanel.class, i18nKey, defaultLabel));
        return jButton;
    }

    
    private int reverseIndex(int row) {
        return jtLayers.getRowCount() - row -1;
    }
    
    /**
     */
    private void moveUp() {
        int row = jtLayers.getSelectedRow();
        if (row <= 0) {
            return;
        }
        // The order is not the same in wmsList nad table
        Collections.swap(tableModel.getWmsList(), reverseIndex(row), reverseIndex(row - 1));
        tableModel.fireTableDataChanged();
        jtLayers.getSelectionModel().setLeadSelectionIndex(row - 1);
    }

    /**
     */
    private void moveDown() {
        int row = jtLayers.getSelectedRow();
        if ((row < 0) || (row >= (jtLayers.getRowCount() - 1))) {
            return;
        }
        // The order is not the same in wmsList nad table
        Collections.swap(tableModel.getWmsList(), reverseIndex(row), reverseIndex(row + 1));
        tableModel.fireTableDataChanged();
        jtLayers.getSelectionModel().setLeadSelectionIndex(row + 1);
    }

    /**
     */
    private void invert() {
        if (jtLayers.getRowCount() == 0) {
            return;
        }
        int row = jtLayers.getSelectedRow();
        Collections.reverse(tableModel.getWmsList());
        tableModel.fireTableDataChanged();
        if (row > -1) {
            jtLayers.getSelectionModel().setLeadSelectionIndex(jtLayers.getRowCount() - row - 1);
        }
    }

    /**
     * @param wmsList
     */
    public void setWmsList(List<BasicWmsLayer> wmsList) {
        tableModel.setWmsList(wmsList);
        tableModel.fireTableDataChanged();
        setButtonStates();
    }

    //~ Classes --------------------------------------------------------------------------------------------------------

    /**
     * @version $Header: /cvs/CVS_LEBA/external/openmap/openmap/src/openmap/com/bbn/openmap/plugin/wms/WmsLayersPanel.java,v 1.4 2006/04/27 05:58:25 pitek Exp $
     * @author
     */
    public class ComboBoxEditor extends DefaultCellEditor {

        //~ Instance variables -----------------------------------------------------------------------------------------

        List<BasicWmsLayer> wmsList;

        //~ Constructors -----------------------------------------------------------------------------------------------

        /**
         * Creates a new DoubleCellRenderer object.
         *
         * @param wmsList
         */
        public ComboBoxEditor(List<BasicWmsLayer> wmsList) {
            super(new JComboBox());
            this.wmsList = wmsList;
        }

        //~ Methods ----------------------------------------------------------------------------------------------------

        /**
         * @param table
         * @param value
         * @param isSelected
         * @param row
         * @param column
         * @return
         */
        public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row,
                int column) {
            JComboBox combo = (JComboBox) editorComponent;
            combo.removeAllItems();
            
            
            for (DefaultWmsLayerStyle style : wmsList.get(reverseIndex(row)).getStyles()) {
                combo.addItem(style.getTitle());
            }
            delegate.setValue(value);
            return editorComponent;
        }

        /**
         * @return
         */
        public int getSelectedIndex() {
            JComboBox combo = (JComboBox) editorComponent;
            return combo.getSelectedIndex();
        }
    }

    /**
     * @version $Header: /cvs/CVS_LEBA/external/openmap/openmap/src/openmap/com/bbn/openmap/plugin/wms/WmsLayersPanel.java,v 1.4 2006/04/27 05:58:25 pitek Exp $
     * @author
     */
    class WmsColumnModel extends DefaultTableColumnModel {

        //~ Instance variables -----------------------------------------------------------------------------------------

        String[] columnNames = {
            i18n.get(WmsColumnModel.class, "visible", "Visible"), i18n.get(WmsColumnModel.class, "title", "Title"),
            i18n.get(WmsColumnModel.class, "name", "Name"), i18n.get(WmsColumnModel.class, "styles", "Styles")
        };

        //~ Methods ----------------------------------------------------------------------------------------------------

        /**
         * @param aColumn
         */
        public void addColumn(javax.swing.table.TableColumn aColumn) {
            aColumn.setHeaderValue(columnNames[aColumn.getModelIndex()]);
            switch (aColumn.getModelIndex()) {
                case 0:
                    aColumn.setPreferredWidth(100);
                    break;
                case 1:
                    aColumn.setPreferredWidth(200);
                    break;
                case 2:
                    aColumn.setPreferredWidth(150);
                    break;
                case 3:
                    aColumn.setPreferredWidth(150);
                    break;
            }
            super.addColumn(aColumn);
        }
    }

    /**
     * @version $Header: /cvs/CVS_LEBA/external/openmap/openmap/src/openmap/com/bbn/openmap/plugin/wms/WmsLayersPanel.java,v 1.4 2006/04/27 05:58:25 pitek Exp $
     * @author
     */
    class WmsTableModel extends AbstractTableModel {

        //~ Instance variables -----------------------------------------------------------------------------------------

        List<BasicWmsLayer> wmsList;

        //~ Constructors -----------------------------------------------------------------------------------------------

        /**
         * Creates a new WmsPanelTableModel object.
         *
         * @param wmsList
         */
        public WmsTableModel(List<BasicWmsLayer> wmsList) {
            this.wmsList = wmsList;
        }

        //~ Methods ----------------------------------------------------------------------------------------------------

        /**
         * @param wmsList
         */
        public void setWmsList(List<BasicWmsLayer> wmsList) {
            this.wmsList = wmsList;
        }

        /**
         * @return
         */
        public List<BasicWmsLayer> getWmsList() {
            return wmsList;
        }

        /**
         * {@inheritDoc}
         */
        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return (columnIndex == COLUMNS.VISIBLE.ordinal()) || (columnIndex == COLUMNS.STYLE.ordinal());
        }

        /**
         * {@inheritDoc}
         */
        public Object getValueAt(int rowIndex, int columnIndex) {
            BasicWmsLayer wms = wmsList.get(reverseIndex(rowIndex));
            switch (COLUMNS.values()[columnIndex]) {
                case VISIBLE:
                    return wms.isVisible();
                case TITLE:
                    return wms.getTitle();
                case NAME:
                    return wms.getName();
                case STYLE:
                    DefaultWmsLayerStyle style = wms.getCurrentStyle();
                    if (style != null) {
                        return wms.getCurrentStyle().getTitle();
                    } else {
                        return null;
                    }
            }
            return null;
        }

        /**
         * @param value
         * @param row
         * @param col
         */
        public void setValueAt(Object value, int row, int col) {
            BasicWmsLayer wms = wmsList.get(reverseIndex(row));
            switch (COLUMNS.values()[col]) {
                case VISIBLE:
                    wms.setVisible((Boolean) value);
                    break;
                case STYLE:
                    // style titles are probably different but ...
                    ComboBoxEditor comboEditor = (ComboBoxEditor) jtLayers.getColumnModel()
                            .getColumn(COLUMNS.STYLE.ordinal()).getCellEditor();
                    wms.setCurrentStyleIndex(comboEditor.getSelectedIndex());
                    break;
            }
        }

        /**
         * {@inheritDoc}
         */
        public int getRowCount() {
            return wmsList.size();
        }

        /**
         * {@inheritDoc}
         */
        public int getColumnCount() {
            return COLUMNS.values().length;
        }

        /**
         * {@inheritDoc}
         */
        public Class getColumnClass(int columnIndex) {
            switch (COLUMNS.values()[columnIndex]) {
                case VISIBLE:
                    return Boolean.class;
                case TITLE:
                    return String.class;
                case NAME:
                    return String.class;
                case STYLE:
                    return String.class;
            }
            return Object.class;
        }
    }
}
