/*
 * $Header: /cvs/CVS_LEBA/external/openmap/openmap/src/openmap/com/bbn/openmap/plugin/wms/WmsPanel.java,v 1.4 2006/04/27 05:58:25 pitek Exp $
 *
 */

package com.bbn.openmap.plugin.wms;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;

import com.bbn.openmap.Environment;
import com.bbn.openmap.I18n;
import com.bbn.openmap.util.propertyEditor.ColorPropertyEditor;

/**
 * Main panel displaying properties of WMS.
 *
 * @author tomk (http://www.ctm.gdynia.pl)
 */
public class WmsPanel extends javax.swing.JPanel {

    //~ Instance variables ---------------------------------------------------------------------------------------------

    protected I18n i18n = Environment.getI18n();
    WMSPlugIn wmsPlugin;
    WmsLayersPanel wmsLayersPanel;

    JDialog serverDialog;

    JComboBox jComboBoxImageFormat;
    JCheckBox jCheckBoxBackground;
    ColorPropertyEditor colorEditor;

    // Variables declaration - do not modify
    private javax.swing.JButton jButtonHide;
    private javax.swing.JButton jButtonQuery;
    private javax.swing.JButton jButtonRefresh;
    private javax.swing.JButton jButtonServer;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    // End of variables declaration

    //~ Constructors ---------------------------------------------------------------------------------------------------

    /**
     * Creates new form WmsPanel
     *
     * @param wmsPlugin
     */
    public WmsPanel(WMSPlugIn wmsPlugin) {
        this.wmsPlugin = wmsPlugin;
        initComponents();
        JPanel displayPropertiesPanel = createDisplayPropertiesPanel();
        jPanel1.add(displayPropertiesPanel);

        setPluginData();
        wmsLayersPanel = new WmsLayersPanel(wmsPlugin.getWmsLayers());
        jPanel2.add(wmsLayersPanel);
        localize();
    }

    //~ Methods --------------------------------------------------------------------------------------------------------

    /**
     */
    public void updatePluginData() {
        wmsPlugin.setImageFormat(WMSPlugIn.IMAGE_PREFIX + (String) jComboBoxImageFormat.getSelectedItem());
        wmsPlugin.setTransparent(Boolean.toString(!jCheckBoxBackground.isSelected()));
        wmsPlugin.setBackgroundColor("0X" + colorEditor.getAsText().substring(2));
        return;
    }

    /**
     */
    public void setPluginData() {
        jComboBoxImageFormat.removeAllItems();
        for (String item : wmsPlugin.getImageFormats()) {
            if (item.indexOf(WMSPlugIn.IMAGE_PREFIX) == 0) {
                item = item.substring(WMSPlugIn.IMAGE_PREFIX.length());
            }            
            jComboBoxImageFormat.addItem(item);
        }
        String imageFormat = wmsPlugin.getImageFormat();
        if (imageFormat.indexOf(WMSPlugIn.IMAGE_PREFIX) == 0) {
            imageFormat = imageFormat.substring(WMSPlugIn.IMAGE_PREFIX.length());
        }
        jComboBoxImageFormat.setSelectedItem(imageFormat);
        jCheckBoxBackground.setSelected("FALSE".equals(wmsPlugin.getTransparent().toUpperCase()));
        colorEditor.setValue("FF" + wmsPlugin.getBackgroundColor().substring(2));
    }

    /**
     * @param wmsPlugin
     */
    private void openServerDialog(final WMSPlugIn wmsPlugin) {
        String title = i18n.get(WmsPanel.class, "title", "Server");
        Window w = SwingUtilities.getWindowAncestor(this);
        if (w instanceof JDialog) {
            serverDialog = new JDialog((JDialog) w, title, true);
        } else if (w instanceof JFrame) {
            serverDialog = new JDialog((JFrame) w, title, true);
        } else {
            serverDialog = new JDialog((JDialog)null, title, true);
        }
        ActionListener closeAction = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                serverDialog.setVisible(false);
                serverDialog.dispose();

                setPluginData();
                wmsLayersPanel.setWmsList(wmsPlugin.getWmsLayers());
            }
        };
        ServerPanel serverPanel = new ServerPanel(wmsPlugin, closeAction);
        serverDialog.getContentPane().add(serverPanel);
        serverDialog.pack();
        Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
        int x = (d.width - serverDialog.getWidth()) / 2;
        int y = (d.height - serverDialog.getHeight()) / 2;
        serverDialog.setLocation((x < 0) ? 0 : x, (y < 0) ? 0 : y);
        serverDialog.setVisible(true);
    }

    /**
     */
    private void localize() {
        jButtonServer.setText(i18n.get(WmsPanel.class, "server", "Server..."));
        jButtonQuery.setText(i18n.get(WmsPanel.class, "query", "Query..."));
        jButtonRefresh.setText(i18n.get(WmsPanel.class, "refresh", "Refresh"));
        jButtonHide.setText(i18n.get(WmsPanel.class, "hide", "Hide"));

        jPanel1.setBorder(BorderFactory.createTitledBorder(
                i18n.get(WmsPanel.class, "displayProperties", "Display properties")));
        jPanel2.setBorder(BorderFactory.createTitledBorder(i18n.get(WmsPanel.class, "layers", "Warstwy")));
    }

    /**
     * @return
     */
    private JPanel createDisplayPropertiesPanel() {
        JPanel displayPropertiesPanel = new JPanel();

        JLabel jLabelImageFormat = new JLabel(i18n.get(WmsPanel.class, "imageFormat", "Image format") + ": ");
        displayPropertiesPanel.add(jLabelImageFormat);

        jComboBoxImageFormat = new JComboBox();
        displayPropertiesPanel.add(jComboBoxImageFormat);

        jCheckBoxBackground = new JCheckBox("  " + i18n.get(WmsPanel.class, "background", "Background") + ": ");
        jCheckBoxBackground.setHorizontalTextPosition(SwingConstants.LEADING);
        displayPropertiesPanel.add(jCheckBoxBackground);

        JLabel jLabelBackgroundColor = new JLabel("  "
                + i18n.get(WmsPanel.class, "backgroundColor", "Background color") + ": ");
        displayPropertiesPanel.add(jLabelBackgroundColor);

        colorEditor = new ColorPropertyEditor();
        displayPropertiesPanel.add(colorEditor.getCustomEditor());

        displayPropertiesPanel.setLayout(new BoxLayout(displayPropertiesPanel, BoxLayout.X_AXIS));
        return displayPropertiesPanel;
    }

    /**
     * This method is called from within the constructor to initialize the form. WARNING: Do NOT modify this code. The
     * content of this method is always regenerated by the Form Editor.
     */
    private void initComponents() {
        jPanel1 = new javax.swing.JPanel();
        jPanel2 = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        jButtonServer = new javax.swing.JButton();
        jButtonQuery = new javax.swing.JButton();
        jButtonRefresh = new javax.swing.JButton();
        jButtonHide = new javax.swing.JButton();

        setLayout(new java.awt.BorderLayout());

        jPanel1.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        add(jPanel1, java.awt.BorderLayout.NORTH);

        jPanel2.setLayout(new java.awt.BorderLayout());

        add(jPanel2, java.awt.BorderLayout.CENTER);

        jPanel3.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        jButtonServer.setText("Server...");
        jButtonServer.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    jButtonServerActionPerformed(evt);
                }
            });

        jPanel3.add(jButtonServer);

        jButtonQuery.setText("Query...");
        jButtonQuery.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    jButtonQueryActionPerformed(evt);
                }
            });

        jPanel3.add(jButtonQuery);

        jButtonRefresh.setText("Refresh");
        jButtonRefresh.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    jButtonRefreshActionPerformed(evt);
                }
            });

        jPanel3.add(jButtonRefresh);

        jButtonHide.setText("Hide");
        jButtonHide.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    jButtonHideActionPerformed(evt);
                }
            });

        jPanel3.add(jButtonHide);

        add(jPanel3, java.awt.BorderLayout.SOUTH);
    }

    /**
     * @param evt
     */
    private void jButtonHideActionPerformed(java.awt.event.ActionEvent evt) {
        Window w = SwingUtilities.getWindowAncestor(this);
        if (w != null) {
            w.setVisible(false);
        }
    }

    /**
     * @param evt
     */
    private void jButtonRefreshActionPerformed(java.awt.event.ActionEvent evt) {
        updatePluginData();
        wmsPlugin.refresh();
    }

    /**
     * @param evt
     */
    private void jButtonQueryActionPerformed(java.awt.event.ActionEvent evt) {
        updatePluginData();
        wmsPlugin.viewQuery(SwingUtilities.getWindowAncestor(this));
    }

    /**
     * @param evt
     */
    private void jButtonServerActionPerformed(java.awt.event.ActionEvent evt) {
        openServerDialog(wmsPlugin);
    }

}
