/*
 * $Header: /cvs/CVS_LEBA/external/openmap/openmap/src/openmap/com/bbn/openmap/plugin/wms/WmsXmlParser.java,v 1.3 2006/04/27 05:58:25 pitek Exp $
 *
 */

package com.bbn.openmap.plugin.wms;

import java.util.List;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import com.bbn.openmap.wms.DefaultWmsLayerStyle;

/**
 * <p>Description: Simple XML parser that tries to get a list of unique layer names It could be extended to get more
 * capabilities...</p>
 *
 * @author Bart Jourquin FUCaM-GTM
 * @author tomk (http://www.ctm.gdynia.pl)
 */
public class WmsXmlParser extends DefaultHandler {

    //~ Instance variables ---------------------------------------------------------------------------------------------

    private String xmlKey;

    private List<BasicWmsLayer> wmsList;
    private List<String> imageFormats;
    private BasicWmsLayer currentWMS;
    private DefaultWmsLayerStyle currentStyle;
    private String styleName;

    //~ Constructors ---------------------------------------------------------------------------------------------------

    /**
     * The parsed data will be stored in the properties, with a given prefix
     *
     * @param wmsList
     * @param imageFormats
     */
    public WmsXmlParser(List<BasicWmsLayer> wmsList, List<String> imageFormats) {
        this.wmsList = wmsList;
        this.imageFormats = imageFormats;
    }

    //~ Methods --------------------------------------------------------------------------------------------------------

    /**
     * Stores a new unique layer name in the properties. See Xerces's documentation for more details on how the parsing
     * of an XML formatted stream works.
     *
     * @param ch char[]
     * @param start int
     * @param length int
     * @throws SAXException
     */
    public void characters(char[] ch, int start, int length) throws SAXException {
        String s = new String(ch, start, length);
        if (ch[0] == '\n') {
            return;
        }

        if (xmlKey.indexOf("WMT_MS_Capabilities.Capability.Layer.") != -1) {
            // We assume the first entry is "Name"
            // todo: the case without name
            if (xmlKey.endsWith("Layer.Name")) {
                currentWMS = new BasicWmsLayer();
                currentWMS.setName(s);
                wmsList.add(currentWMS);
            } else if (xmlKey.endsWith("Layer.Title")) {
                if (currentWMS.getTitle() == null) {
                    currentWMS.setTitle(s);
                }
            } else if (xmlKey.endsWith("Layer.Abstract")) {
                currentWMS.setAbstract(s);
            } else if (xmlKey.endsWith("Style.Name")) {
                // We assume the first entry in Style is "Name"
                styleName = s;
            } else if (xmlKey.endsWith("Style.Title")) {
                currentStyle = currentWMS.addStyle(styleName, s, null);
            }
        }

        if (xmlKey.endsWith("Format") && (s.indexOf(WMSPlugIn.IMAGE_PREFIX) == 0)) {
            imageFormats.add(s);
        }
    }

    /**
     * Tells to the XML parser that the keys are separated with a dot. See Xerce's documentation for more details on how
     * the parsing of an XML stream works
     *
     * @param uri String
     * @param localName String
     * @param qName String
     * @throws SAXException
     */
    public void endElement(String uri, String localName, String qName) throws SAXException {
        int i = xmlKey.lastIndexOf('.');
        if (i != -1) {
            xmlKey = xmlKey.substring(0, i);
        }
    }

    /**
     * See Xerce's documentation for more details on how the parsing of an XML stream works
     *
     * @throws SAXException
     */
    public void startDocument() throws SAXException {
        xmlKey = new String("");
    }

    /**
     * Tells to the XML parser that the keys are separated with a dot. See Xerce's documentation for more details on how
     * the parsing of an XML stream works
     *
     * @param uri String
     * @param localName String
     * @param qName String
     * @param attributes Attributes
     * @throws SAXException
     */
    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
        if (xmlKey.length() > 0) {
            xmlKey += ".";
        }
        xmlKey += localName;
    }
}
