/*
 * $Header: /cvs/CVS_LEBA/external/openmap/openmap/src/openmap/com/bbn/openmap/wms/DefaultLayerAdapter.java,v 1.2 2006/03/30 07:38:03 wachu Exp $
 *
 * Copyright 2001-2005 OBR Centrum Techniki Morskiej, All rights reserved.
 *
 */
package com.bbn.openmap.wms;

import com.bbn.openmap.Layer;

/**
 * @version $Header: /cvs/CVS_LEBA/external/openmap/openmap/src/openmap/com/bbn/openmap/wms/DefaultLayerAdapter.java,v 1.2 2006/03/30 07:38:03 wachu Exp $
 * @author pitek
 * Layer Attributes for OGC WMS ver 1.1.1
 * default value for every attribute is 0 
 */
public class DefaultLayerAdapter implements IWmsLayer {

    //~ Static variables/initializers ----------------------------------------------------------------------------------

    private static final IWmsLayerStyle[] defaultStyle = new IWmsLayerStyle[1];

    static {
        defaultStyle[0] = new DefaultWmsLayerStyle("default", "Default style", null);
    }

    //~ Instance variables ---------------------------------------------------------------------------------------------

    protected final Layer layer;

    //~ Constructors ---------------------------------------------------------------------------------------------------

    /**
     * Creates a new instance of DefaultLayerAdapter
     * 
     * @param layer
     */
    public DefaultLayerAdapter(Layer layer) {
        this.layer = layer;
    }

    //~ Methods --------------------------------------------------------------------------------------------------------

    /**
     * @return
     * attribute queryable [0, 1] 
     * 0: not queryable.
     * 1: queryable.
     */
    public boolean isQueryable() {
        return false;
    }

    /**
     * @return
     * attribute opaque [0, 1] 
     * 0: map data represents vector features that probably do not completely fill space.
     * 1: map data are mostly or completely opaque.
     */
    public boolean isOpaque() {
        return false;
    }

    /**
     * @return attribute 
     * noSubsets [0, 1]
     * 0: WMS can map a subset of the full bounding box.
     * 1: WMS can only map the entire bounding box.
     */
    public boolean isNoSubsets() {
        return false;
    }

    /**
     * @return
     * attribute cascaded [0, positive integer]
     * 0: layer has not been retransmitted by a Cascading Map Server.
     * n: layer has been retransmitted n times.
     */
    public boolean isCascaded() {
        return false;
    }

    /**
     * @return
     */
    public String getTitle() {
        return layer.getName();
    }

    /**
     * @return
     */
    public IWmsLayerStyle[] getStyles() {
        return defaultStyle;
    }

    /**
     * @return
     */
    public String getName() {
        return layer.getPropertyPrefix();
    }

    /**
     * @return
     */
    public String getKeywordList() {
        return null;
    }

    /**
     * @return
     * fixedWidth [0, positive integer]
     * 0: WMS can resize map to arbitrary width.
     * nonzero: map has a fixed width that cannot be changed by the WMS.
     */
    public int getFixedWidth() {
        return 0;
    }

    /**
     * @return
     * fixedHeight [0, positive integer]
     * 0: WMS can resize map to arbitrary height.
     * nonzero: map has a fixed height that cannot be changed by the WMS.
     */
    public int getFixedHeight() {
        return 0;
    }

    /**
     * @return
     */
    public String getAbstract() {
        return layer.getName();
    }

    /**
     * @param name
     * @throws IllegalArgumentException
     */
    public void setStyle(String name) {
        if (!isStyleSupported(name)) {
            throw new IllegalArgumentException("Unsupported style " + name);
        }
    }

    /**
     * @param name
     * @return
     */
    public boolean isStyleSupported(String name) {
        for (IWmsLayerStyle style : getStyles()) {
            if (style.getName().equals(name)) {
                return true;
            }
        }
        return false;
    }

    /**
     */
    public void setDefaultStyle() {
        // DO NOTHING
    }
     
}
