/*
 * $Header: /cvs/CVS_LEBA/external/openmap/openmap/src/openmap/com/bbn/openmap/wms/MapServer.java,v 1.1 2006/03/21 10:27:54 tomrak Exp $
 *
 * Copyright 2001-2005 OBR Centrum Techniki Morskiej, All rights reserved.
 *
 */
package com.bbn.openmap.wms;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Map;
import java.util.HashMap;
import java.util.Properties;
import java.net.URLDecoder;

import com.bbn.openmap.util.PropUtils;
import com.bbn.openmap.Layer;
import com.bbn.openmap.LayerHandler;
import com.bbn.openmap.PropertyHandler;
import com.bbn.openmap.image.MapRequestFormatException;
import com.bbn.openmap.layer.util.http.*;
import com.bbn.openmap.util.ArgParser;
import com.bbn.openmap.util.Debug;
import com.bbn.openmap.util.PropertyStringFormatException;

/**
 * @version $Header: /cvs/CVS_LEBA/external/openmap/openmap/src/openmap/com/bbn/openmap/wms/MapServer.java,v 1.1 2006/03/21 10:27:54 tomrak Exp $
 * @author wachu
 * @author pitek
 */
public class MapServer implements HttpRequestListener {


    protected HttpServer httpd;
    protected WmsRequestHandler wmsRequestHandler;


    /**
     * Creates a new MapServer object.
     * 
     * @param port
     * @param asDeamon
     * @param props
     * @param mapLayerHandler
     * @param wmsLayersMap
     * @throws IOException
     */
    public MapServer(int port,  boolean asDeamon, WmsRequestHandler wmsRequestHandler ) throws IOException {

        httpd = new HttpServer(port, asDeamon);
        httpd.addHttpRequestListener(this);
            
        this.wmsRequestHandler = wmsRequestHandler; 
    }

    
    protected Properties convertRequestToProps(String request)
            throws MapRequestFormatException {
        try {
            // Convert any %XX to the real ASCII value.
            request = URLDecoder.decode(request, "UTF-8");
            Properties requestProperties = PropUtils.parsePropertyList(request, 
                PropUtils.httpRequestPropertySeparators,
                PropUtils.httpRequestKeyValueSeparators, true);
            if (Debug.debugging("ms")) {
                Debug.output("MapServer: parsed request " + requestProperties);
            }
            return requestProperties;
        } catch (PropertyStringFormatException psfe) {
            throw new MapRequestFormatException(psfe.getMessage());
        } catch (Exception e) {
            throw new MapRequestFormatException(e.getMessage());
        }
    }
    

    /**
     * @param e
     * @throws java.io.IOException
     */
    public void httpRequest(HttpRequestEvent e) throws java.io.IOException {
        String request = e.getRequest();

        if (Debug.debugging("ms")) {
            Debug.output("MS: Handling request - \"" + request + "\"");
        }

        if (request.charAt(0) == '/') {
            request = request.substring(0, request.length());
        }

        int index = 0;
        index = request.indexOf('?');
        if (index != -1) { // GET Request
            request = request.substring(index + 1, request.length());
            Debug.message("ms", "MS: GET Request received");
        } else {
            Debug.message("ms", "MS: Probably a POST Request received");
        }

        HttpResponse httpResponse = new HttpResponse(e.getOutputStream());
        try {
            Properties requestProperties = convertRequestToProps(request);
            wmsRequestHandler.handleRequest(requestProperties,httpResponse );
        } catch (IOException ioe) {
            Debug.error("MS: caught IOException - \n" + ioe.getMessage());
        } catch (MapRequestFormatException exception) {
            String message = "MapServer encountered a problem with your request:\n\n" + exception.getMessage();
            Debug.error("MS: caught MapRequestFormatException - \n" + message);
            httpResponse.writeHttpResponse(HttpConnection.CONTENT_PLAIN, message);
        }
    }

    /**
     * @param layer
     */
    public void addLayer(Layer layer) {
        wmsRequestHandler.updateLayers();
    }

    /**
     * @param args
     */
    public static void main(String[] args) {
        Debug.init();
        try {
            ArgParser ap = new ArgParser("MapServer");
            ap.add("properties", "A URL to use to set the properties for the ImageServer.", 1);
            ap.add("port", "The port to listen for new map image requests on. (Default 9999)", 1);
            ap.add("verbose", "Print action messages.");
            ap.add("test", "Create a test default image.");

            if (!ap.parse(args)) {
                ap.printUsage();
                System.exit(0);
            }

            String[] proparg;
            PropertyHandler propHandler;
            proparg = ap.getArgValues("properties");
            if (proparg != null) {
                propHandler = new PropertyHandler(proparg[0]);
            } else {
                propHandler = new PropertyHandler();
            }

            String[] varg = ap.getArgValues("verbose");
            if (varg != null) {
                Debug.put("ms");
                Debug.put("imageserver");
            }

            int port = 9999;
            String[] portarg = ap.getArgValues("port");
            if (portarg != null) {
                port = Integer.parseInt(portarg[0]);
            }
            WmsRequestHandler wmsRequestHandler = new WmsRequestHandler(port, "/openmap", propHandler.getProperties());
            MapServer ms = new MapServer(port, false, wmsRequestHandler);
            
        } catch (WMSException wmse) {
            System.err.println("WMSException");
            wmse.printStackTrace();
        } catch (MalformedURLException murle) {
            System.err.println("Bad URL path to properties file.");
            murle.printStackTrace();
        } catch (IOException ioe) {
            System.err.println("Unable to start http server:");
            ioe.printStackTrace();
        }
    }
}
